@@ -269,7 +269,6 @@ sub connection_manager {
     my %manager;
     my %hash2fd;
     my %wait_read;
-    my $term_received = 0;
     $self->{sockets} = {};
     $self->{fdsend_queue} = [];
 
@@ -277,21 +276,34 @@ sub connection_manager {
 
     my $cv = AE::cv;
     my $close_all = 0;
+    my $i = 0;
     my $sig2;$sig2 = AE::signal 'USR1', sub {
         my $t;$t = AE::timer 0, 1, sub {
-            return unless $close_all;
+            if ( not $close_all && $i < 4 ) {
+                return;
+            }
+            elsif ( $i >= 4 ) {
+                warn "Worker $worker_pid sockets did not close in time. Forcing abort.";
+            }
             undef $t;
             kill 'TERM', $worker_pid;
+            ++$i;
+            my $j = 0;
             my $t2;$t2 = AE::timer 0, 1, sub {
                 my $kid = waitpid($worker_pid, WNOHANG);
-                return if $kid >= 0;
+                ++$j;
+                if ( $kid >= 0 && $j < 60 ) {
+                    return;
+                }
+                elsif ( $j >= 60 ) {
+                    warn "Worker $worker_pid still here after $j iterations. Forcing abort.";
+                }
                 undef $t2;
                 $cv->send;
             };
         };
     };
     my $sig;$sig = AE::signal 'TERM', sub {
-        $term_received++;
         kill 'USR1', $worker_pid; #stop accept
         my $t;$t = AE::timer 0, 1, sub {
             my $time = time;
